/*
 
  copyright (c) 2004, Michal Salaban <emes@pld-linux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: log.c,v 1.5 2004/02/26 20:11:09 emes Exp $
 
*/

#include <stdio.h>
#include <syslog.h>
#include "xalloc.h"
#include "parseopt/lex.h"
#include "getstats.h"
#include "conditions.h"
#include "log.h"

char * stats2str (int *sta)
{
    char    *res = xalloc (NULL, LOGLINEBUF);
    char    *tmp = xalloc (NULL, LOGVARBUF);
    int     i;

    *res = '\0';

    for (i = 0; i < STA_LAST; i++) {
        if (*(sta + i) == 1) {
            switch (stat[i].type) {
                case DOUBLE:
                    snprintf (tmp,
                              LOGVARBUF - 1,
                              "%s = %0.2f, ",
                              stat[i].label,
                              stat[i].val.dbl
                             );
                    break;
                case SIZE:
                    snprintf (tmp,
                              LOGVARBUF - 1,
                              "%s = %d, ",
                              stat[i].label,
                              stat[i].val.siz
                             );
                    break;
            }

            strcat (res, tmp);
        }
    }

    free (tmp);

    tmp = res + strlen (res) - 1;
    while ((*tmp == ',') || (*tmp == ' ')) {
        *tmp = '\0';
        tmp --;
    }
    
    return res;
}

void log_start (int rules)
{
    openlog ("sysfence",
            LOG_NDELAY | LOG_PID | LOG_CONS,
            LOG_DAEMON);
    
    syslog (LOG_INFO, "loaded %d rules", rules);
}

void log_rulehit (ruleset *rule)
{
    char    *stattxt = stats2str (&(rule->showstat[0]));    
    
    if (rule->name != NULL)
        syslog (LOG_WARNING,
                "%s: %s",
                rule->name,
                stattxt
               );
    else
        syslog (LOG_WARNING,
                "rule hit: %s",
                stattxt
               );

    free (stattxt);
        
    /*
    if (rule->name != NULL) {
        syslog (LOG_WARNING,
                "%s: %s=%0.2f %s=%0.2f %s=%0.2f, "
                "%s=%d %s=%d, %s=%d %s=%d",
                rule->name,
                stat[STA_LA1].label, stat[STA_LA1].val.dbl,
                stat[STA_LA5].label, stat[STA_LA5].val.dbl,
                stat[STA_LA15].label, stat[STA_LA15].val.dbl,
                stat[STA_MFREE].label, stat[STA_MFREE].val.siz,
                stat[STA_MUSED].label, stat[STA_MUSED].val.siz,
                stat[STA_SFREE].label, stat[STA_SFREE].val.siz,
                stat[STA_SUSED].label, stat[STA_SUSED].val.siz
                );
    } else {
        syslog (LOG_WARNING,
                "rule hit: %s=%0.2f %s=%0.2f %s=%0.2f, "
                "%s=%d %s=%d, %s=%d %s=%d",
                stat[STA_LA1].label, stat[STA_LA1].val.dbl,
                stat[STA_LA5].label, stat[STA_LA5].val.dbl,
                stat[STA_LA15].label, stat[STA_LA15].val.dbl,
                stat[STA_MFREE].label, stat[STA_MFREE].val.siz,
                stat[STA_MUSED].label, stat[STA_MUSED].val.siz,
                stat[STA_SFREE].label, stat[STA_SFREE].val.siz,
                stat[STA_SUSED].label, stat[STA_SUSED].val.siz
                );
    }
    */
}

void log_end ()
{
    syslog (LOG_INFO, "exiting");

    closelog ();
}

/* $Id: log.c,v 1.5 2004/02/26 20:11:09 emes Exp $ */
